package imagemed.layout.validator;

import imagemed.types.enumeration.IMTypebe;
import imagemed.util.LocalizedMessage;
import java.text.Format;

import java.util.Properties;


public class DefaultDoctypeValidator extends DBValidator {

	@Override
	protected void preValidate() {

		setMandatoryError( "DTNA" );
		setMandatoryError( "DTUI" );
	}


	@Override
	public void validate() {

		mustBeUnique( "DTNA", m_doctypePage );
		mustBeUnique( "DTUI", m_doctypePage );

		// error for changing id=0
		String actValue = m_rootRec.getCurrentValue( "DTUI", "" );
		String oldValue = m_rootRec.getOldValue( "DTUI", "" );
		if ( "0".equals( oldValue ) && !"0".equals( actValue ) ) {
			m_errors.add( LocalizedMessage.getSimpleMessage( "DEFAULT_DOCTYPE_VALIDATOR_ZERO_ID_UNDEFINED_CHANGED" ) );
		}

		// Don't give Study behavior doctype as default for a Patient/Order
		String behav = m_rootRec.getCurrentValue( "DTBE", "" );
		if ( IMTypebe.STUDY.equals( behav ) ) {
			if ( "yes".equals( m_rootRec.getCurrentValue( "DTOD", "no" ) ) ||
					"yes".equals( m_rootRec.getCurrentValue( "DTPD", "no" ) ) ||
					"yes".equals( m_rootRec.getCurrentValue( "DTOU", "no" ) ) ||
					"yes".equals( m_rootRec.getCurrentValue( "DTPU", "no" ) ) ) {
				m_errors.add( LocalizedMessage.getSimpleMessage( "DEFAULT_DOCTYPE_VALIDATOR_CANNOT_SET_DOCUMENT" ) );
			}
		}

		// warning for 2 or more records for the same group
		Properties groupnames = new Properties();
		for ( RecordIterator group = getIterator( m_doctypeGroupTable ); group.next(); ) {
			String name = group.getValue( "DGNA" );
			if ( name != null ) {
				if ( "Exists".equals( groupnames.getProperty( name ) ) ) {
					m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_DOCTYPE_VALIDATOR_TWO_OR_MORE_GROUP", new Format[]{ null }, new Object[]{ name }, false ) );
					groupnames.setProperty( name, "Warninged" );
				}
				else if ( groupnames.getProperty( name ) == null ) {
					groupnames.setProperty( name, "Exists" );
				}
			}
		}

		// warning for 2 or more records for the same user
		Properties usernames = new Properties();
		for ( RecordIterator user = getIterator( m_doctypeUserTable ); user.next(); ) {
			String name = user.getValue( "DUNA" );
			if ( name != null ) {
				if ( "Exists".equals( usernames.getProperty( name ) ) ) {
					m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_DOCTYPE_VALIDATOR_TWO_OR_MORE_USER", new Format[]{ null }, new Object[]{ name }, false )  );
					usernames.setProperty( name, "Warninged" );
				}
				else if ( usernames.getProperty( name ) == null ) {
					usernames.setProperty( name, "Exists" );
				}
			}
		}

		// If this is the default of a page, set every other doctype not default of that page 
		if ( "yes".equals( m_rootRec.getCurrentValue( "DTQD", "no" ) ) ) {
			setEachRecordsValue( m_doctypeTable, "DTQD", "no", "DTUI", actValue );
		}
		if ( "yes".equals( m_rootRec.getCurrentValue( "DTAD", "no" ) ) ) {
			setEachRecordsValue( m_doctypeTable, "DTAD", "no", "DTUI", actValue );
		}
		if ( "yes".equals( m_rootRec.getCurrentValue( "DTOD", "no" ) ) ) {
			setEachRecordsValue( m_doctypeTable, "DTOD", "no", "DTUI", actValue );
		}
		if ( "yes".equals( m_rootRec.getCurrentValue( "DTPD", "no" ) ) ) {
			setEachRecordsValue( m_doctypeTable, "DTPD", "no", "DTUI", actValue );
		}

		// The same for Upload 
		if ( "yes".equals( m_rootRec.getCurrentValue( "DTQU", "no" ) ) ) {
			setEachRecordsValue( m_doctypeTable, "DTQU", "no", "DTUI", actValue );
		}
		if ( "yes".equals( m_rootRec.getCurrentValue( "DTAU", "no" ) ) ) {
			setEachRecordsValue( m_doctypeTable, "DTAU", "no", "DTUI", actValue );
		}
		if ( "yes".equals( m_rootRec.getCurrentValue( "DTOU", "no" ) ) ) {
			setEachRecordsValue( m_doctypeTable, "DTOU", "no", "DTUI", actValue );
		}
		if ( "yes".equals( m_rootRec.getCurrentValue( "DTPU", "no" ) ) ) {
			setEachRecordsValue( m_doctypeTable, "DTPU", "no", "DTUI", actValue );
		}

	}

}
