package imagemed.layout.validator;

import imagemed.exception.IMException;
import imagemed.imtables.OrderScheduleModeTopCell;
import imagemed.layout.LayoutPart;
import imagemed.layout.datasource.pointer.RecordPointer;
import imagemed.types.enumeration.IMTypeos;
import imagemed.util.*;

import java.util.*;

public class DefaultschDeviceValidator extends DBValidator {

	@Override
	protected void preValidate() {
		setMandatoryError( "0DNA" );
		setMandatoryError( "0DFA" );
		setMandatoryError( "0DTY" );

		//set Days Operating from/to non-editable if device is not operating on that day
		/*		
				String operatingDays = getValue( "0DOD" );
				if( operatingDays != null ) {
					if( operatingDays.indexOf( '1' ) == -1 ) {
						setNonEditable( "0DF1" );
						setNonEditable( "0DT1" );
					}
					if( operatingDays.indexOf( '2' ) == -1 ) {
						setNonEditable( "0DF2" );
						setNonEditable( "0DT2" );
					}
					if( operatingDays.indexOf( '3' ) == -1 ) {
						setNonEditable( "0DF3" );
						setNonEditable( "0DT3" );
					}
					if( operatingDays.indexOf( '4' ) == -1 ) {
						setNonEditable( "0DF4" );
						setNonEditable( "0DT4" );
					}
					if( operatingDays.indexOf( '5' ) == -1 ) {
						setNonEditable( "0DF5" );
						setNonEditable( "0DT5" );
					}
					if( operatingDays.indexOf( '6' ) == -1 ) {
						setNonEditable( "0DF6" );
						setNonEditable( "0DT6" );
					}
					if( operatingDays.indexOf( '7' ) == -1 ) {
						setNonEditable( "0DF7" );
						setNonEditable( "0DT7" );
					}
				}
		*/

		//Refresh - Set fields
		String refreshingType = getRefreshType();

		if ( REFRESHTYPE_FIELDCHANGE.equals( refreshingType ) ) {
			String refreshingColid = getRefreshingColid();

			//Procedure selected -> Time Needed default from Procedure
			if ( "0EPI".equals( refreshingColid ) ) {
				RecordIterator devproc = getRefreshingRecords();
				if ( !devproc.isEmpty() ) {
					while ( devproc.next() ) {
						for ( RecordIterator proc = devproc.getIterator( m_schProcedureTable ); proc.next(); ) {
							String timeNeeded = proc.getValue( "0HTI" );
							if ( timeNeeded != null && !"".equals( timeNeeded ) ) {
								devproc.setValue( "0ETI", timeNeeded );
							}
						}
					}
				}
			}

			//Operating days, from, to Facility->Device
			if ( "0DFA".equals( refreshingColid ) ) {
				for ( RecordIterator facility = getIterator( m_schFacilityTable ); facility.next(); ) {
					//there will be 1

					String[][] colids = { { "0FOD", "0DOD" }, { "0FOF", "0DOF" }, { "0FOT", "0DOT" },
							{ "0FF1", "0DF1" }, { "0FT1", "0DT1" },
							{ "0FF2", "0DF2" }, { "0FT2", "0DT2" },
							{ "0FF3", "0DF3" }, { "0FT3", "0DT3" },
							{ "0FF4", "0DF4" }, { "0FT4", "0DT4" },
							{ "0FF5", "0DF5" }, { "0FT5", "0DT5" },
							{ "0FF6", "0DF6" }, { "0FT6", "0DT6" },
							{ "0FF7", "0DF7" }, { "0FT7", "0DT7" } };
					copyValues( colids, facility, null );
				}
			}

			//Operating From -> days Operating From
			if ( "0DOF".equals( refreshingColid ) ) {
				String[][] colids = { { "0DOF", "0DF1" }, { "0DOF", "0DF2" }, { "0DOF", "0DF3" }, { "0DOF", "0DF4" }, { "0DOF", "0DF5" }, { "0DOF", "0DF6" }, { "0DOF", "0DF7" } };
				copyValues( colids, null, null );
			}

			//Operating To -> days Operating To
			if ( "0DOT".equals( refreshingColid ) ) {
				String[][] colids = { { "0DOT", "0DT1" }, { "0DOT", "0DT2" }, { "0DOT", "0DT3" }, { "0DOT", "0DT4" }, { "0DOT", "0DT5" }, { "0DOT", "0DT6" }, { "0DOT", "0DT7" } };
				copyValues( colids, null, null );
			}
		}
	}

	@Override
	public void validate() {
		setPrimaryWarning( new String[]{ "0DNA", "0DFA", "0DTY", "0DAE" } );
	}


	@Override
	protected void deleteValidate( RecordPointer rp ) {
		if ( rp == null ) {
			super.deleteValidate( rp );
		}
		else {
			//we delete a device-procedure record, check the Orders
			String linkPlus = "";
			Properties extraPars = new Properties();
			//see all orders
			linkPlus = "&cOrderScheduleMode=" + OrderScheduleModeTopCell.SCH_NONE;
			//completed and above doesn't count
			extraPars.setProperty( "0ZST", ":" + IMTypeos.STATE_INPROCESS );

			Set<RecordPointer> records;
			try {
				records = getRecordsReferringTo( LayoutPart.SCHORDERLAYOUT, true, true, extraPars, rp );
			}
			catch ( IMException e ) {
				Helper.defaultCatch( e );
				return;
			}
			if ( records.size() > 0 ) {
				m_errors.add( LocalizedMessage.getSimpleMessage( "DBVALIDATOR_REFERRING_ORDERS_EXIST" ) +
						"  <A HREF='" + getLinkToReferrersOf( LayoutPart.SCHORDERLAYOUT, extraPars, rp ) + linkPlus +
						"'>See these Orders </A>" );
			}

		}

	}
}
