package imagemed.layout.validator;

import imagemed.exception.IMException;
import imagemed.imsession.IMEnvBase;
import imagemed.imtables.OrderScheduleModeTopCell;
import imagemed.layout.LayoutPart;
import imagemed.layout.datasource.pointer.RecordPointer;
import imagemed.types.enumeration.IMTypeos;
import imagemed.util.*;

import java.text.Format;
import java.util.*;


public class DefaultschProcedureValidator extends DBValidator {

	@Override
	protected void preValidate() {
		setMandatoryError( "0HNA" );
		setMandatoryError( "0HTY" );
		setMandatoryError( "0HTI" );

		//Refresh - Set fields
		String refreshingColid = getRefreshingColid();
		String refreshingType = getRefreshType();

		if ( REFRESHTYPE_FIELDCHANGE.equals( refreshingType ) ) {
			//Device selected -> Time Needed default from Procedure
			if ( "0EDI".equals( refreshingColid ) ) {
				RecordIterator devproc = getRefreshingRecords();
				String timeNeeded = getValue( "0HTI" );
				if ( !devproc.isEmpty() && timeNeeded != null && !"".equals( timeNeeded ) ) {
					while ( devproc.next() ) {
						devproc.setValue( "0ETI", timeNeeded );
					}
				}
			}

			//Combined procedures Body part, Time needed, CPT4
			if ( "0CPC".equals( refreshingColid ) ) {
				calcCombinedFields();
			}
		}

		//If a Sub-procedure was deleted, re-calculate Combined procedures Body part, Time needed, CPT4
		else if ( REFRESHTYPE_RECORDDELETE.equals( refreshingType ) ) {
			Helper.DEBUG( "ProcedureValidator", "refreshingColid: " + refreshingColid );
			if ( "0CPP".equals( refreshingColid ) ) {
				calcCombinedFields();
			}
		}
	}


	protected void calcCombinedFields() {
		//Body part  = 'BodyPart'\'BodyPart'
		//Set because we don't want more of the same
		//CPT4 = 'CPT4'\'CPT4' - set
		//Time needed = sum of times of sub-procedures
		String fullBodyPart = "";
		String fullTime = "000000";
		String fullCPT4 = "";
		Set<String> bpSet = new HashSet<String>();
		Set<String> CPT4Set = new HashSet<String>();
		boolean first = true;
		for ( RecordIterator combProc = getIterator( m_schCombinedProcedureTable ); combProc.next(); ) {
			String subprocID = combProc.getValue( "0CPC" );
			RecordIterator proc = getExternalRecord( m_schProcedureTable, subprocID );
			proc.next();
			String bodyPart = proc.getValue( "0HBP" );
			if ( bodyPart != null && !"".equals( bodyPart ) ) {
				bpSet.add( bodyPart );
			}
			String CPT4 = proc.getValue( "0HCP" );
			if ( CPT4 != null && !"".equals( CPT4 ) ) {
				CPT4Set.add( CPT4 );
			}
			String time = proc.getValue( "0HTI" );
			if ( time != null && !"".equals( time ) ) {
				fullTime = addTimes( fullTime, time );
			}
		}
		if ( bpSet.size() > 0 ) {
			for ( Iterator<String> iter = bpSet.iterator(); iter.hasNext(); ) {
				if ( first ) {
					first = false;
				}
				else {
					fullBodyPart += "\\";
				}
				fullBodyPart += iter.next();
			}
		}
		if ( !"".equals( fullBodyPart ) ) {
			setValue( "0HBP", fullBodyPart );
		}

		first = true;
		if ( CPT4Set.size() > 0 ) {
			for ( Iterator<String> iter = CPT4Set.iterator(); iter.hasNext(); ) {
				if ( first ) {
					first = false;
				}
				else {
					fullCPT4 += "\\";
				}
				fullCPT4 += iter.next();
			}
		}
		if ( !"".equals( fullCPT4 ) ) {
			setValue( "0HCP", fullCPT4 );
		}

		if ( !"000000".equals( fullTime ) ) {
			setValue( "0HTI", fullTime );
		}
	}

	@Override
	public void validate() {

		setPrimaryWarning( new String[]{ "0HNA", "0HTY", "0HBP" } );

		String timeNeeded = getValue( "0HTI" );
		if ( !"".equals( timeNeeded ) ) {
			try {
				int timeInt = Integer.parseInt( timeNeeded );
				if ( timeInt <= 0 ) {
					m_errors.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_PROCEDURE_VALIDATOR_MUST_BE_POSITIVE", new Format[]{ null }, new Object[]{ getLabel( "0HTI" ) }, false ) );
				}
			}
			catch ( Throwable th ) {
				m_errors.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_PROCEDURE_VALIDATOR_NOT_VALID_NUMBER", new Format[]{ null }, new Object[]{ getLabel( "0HTI" ) }, false ) );
			}
		}

		//Devices should be selected
		for ( RecordIterator devProc = getIterator( m_schDeviceProcedureTable ); devProc.next(); ) {
			String deviceID = devProc.getValue( "0EDI" );
			if ( deviceID == null || "".equals( deviceID ) ) {
				m_warnings.add( LocalizedMessage.getSimpleMessage( "DEFAULT_SCH_PROCEDURE_VALIDATOR_DEVICE_SHOULD_BE_SELECTED" ) );
			}
		}

		//if I am a Combined Procedure, is there a combined procedure I am a sub-procedure of?
		RecordIterator combProc0 = getIterator( m_schCombinedProcedureTable );
		if ( combProc0.next() ) {
			String procID = getValue( "0HID" );
			Properties parentProcFilters = new Properties();
			parentProcFilters.setProperty( "0CPC", procID );
			MTDBIterator parentProcesses = getMTDBIterator( new String[]{ "0CPP" }, parentProcFilters );
			while ( parentProcesses.next() ) {
				RecordIterator parentProc = getExternalRecord( m_schProcedureTable, parentProcesses.getValue( "0CPP" ) );
				if ( parentProc != null ) {
					parentProc.next();
					String parentProcName = parentProc.getValue( "0HNA" );
					m_errors.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_PROCEDURE_VALIDATOR_COMBINED_PROCEDURE_SUBPROCEDURE", new Format[]{ null }, new Object[]{ parentProcName }, false ) );
				}
			}
		}

		//if I am a Combined Procedure, does all device support all subprocedure?
		for ( RecordIterator combProc = getIterator( m_schCombinedProcedureTable ); combProc.next(); ) {
			String subprocID = combProc.getValue( "0CPC" );
			if ( subprocID != null ) {
				RecordIterator subproc = getExternalRecord( m_schProcedureTable, subprocID );
				subproc.next();
				String subprocName = subproc.getValue( "0HNA" );

				//if I am a Combined Procedure, is there a combined sub-procedure?
				Properties subprocFilters = new Properties();
				subprocFilters.setProperty( "0CPP", subprocID );
				MTDBIterator subSubProc = getMTDBIterator( new String[]{ "0CPC" }, subprocFilters );
				if ( subSubProc.next() ) {
					String child = subSubProc.getValue( "0CPC" );
					if ( child != null && !"".equals( child ) ) {
						Helper.DEBUG( "procedure validator", "0CPC:" + child );
						m_errors.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_PROCEDURE_VALIDATOR_COMBINED_PROCEDURE_CONTAINS_ANOTHER", new Format[]{ null }, new Object[]{ subprocName }, false ) );

					}
				}

				//devproc records connecting to this (main) procedure
				for ( RecordIterator devProc = getIterator( m_schDeviceProcedureTable ); devProc.next(); ) {
					String deviceID = devProc.getValue( "0EDI" );
					boolean foundOnDevice = false;
					String deviceName = "";
					for ( RecordIterator device = devProc.getIterator( m_schDeviceTable ); device.next(); ) {
						//there will be 1
						deviceName = device.getValue( "0DNA" );

						//get devproc records connecting to ths device
						Properties filters = new Properties();
						filters.setProperty( "0EDI", deviceID );
						MTDBIterator allDevProc = getMTDBIterator( new String[]{ "0EPI" }, filters );
						while ( allDevProc.next() ) {
							if ( subprocID.equals( allDevProc.getValue( "0EPI" ) ) ) {
								foundOnDevice = true;
								break;
							}
						}
					}
					if ( !foundOnDevice ) {
						m_warnings.add( LocalizedMessage.getCompoundMessage( "DEFAULT_SCH_PROCEDURE_VALIDATOR_SUBPROCEDURE_NOT_SUPPORTED", new Format[]{ null, null }, new Object[]{ subprocName, deviceName }, false ) );
					}
				}
			}
		}
	}

	@Override
	protected void deleteValidate( RecordPointer rp ) {
		if ( rp == null ) {
			super.deleteValidate( rp );
		}
		else {
			String devprocTable = IMEnvBase.getEnv( "SCHEDULERDEVPROCTBL" );
			if ( devprocTable != null && devprocTable.equals( rp.m_source.m_stable ) ) {
				//we delete a device-procedure record, check the Orders
				String linkPlus = "";
				Properties extraPars = new Properties();
				//see all orders
				linkPlus = "&cOrderScheduleMode=" + OrderScheduleModeTopCell.SCH_NONE;
				//completed and above doesn't count
				extraPars.setProperty( "0ZST", ":" + IMTypeos.STATE_INPROCESS );

				Set<RecordPointer> records;
				try {
					records = getRecordsReferringTo( LayoutPart.SCHORDERLAYOUT, true, true, extraPars, rp );
				}
				catch ( IMException e ) {
					Helper.defaultCatch( e );
					return;
				}
				if ( records.size() > 0 ) {
					m_errors.add( LocalizedMessage.getSimpleMessage( "DBVALIDATOR_REFERRING_ORDERS_EXIST" ) +
							" <A HREF='" + getLinkToReferrersOf( LayoutPart.SCHORDERLAYOUT, extraPars, rp ) + linkPlus +
							"'>See these Orders </A>" );
				}
			}
		}

	}


}
