package imagemed.layout.validator;

import imagemed.exception.IMException;
import imagemed.imsession.IMEnv;
import imagemed.imtables.IMBaseFieldManager;
import imagemed.jsputil.IMAdminToolbox;
import imagemed.jsputil.PluginToolbox;
import imagemed.layout.datasource.DBLDSEntry;
import imagemed.layout.datasource.DBLayoutDataSource;
import imagemed.layout.datasource.LDSRecord;
import imagemed.layout.datasource.pointer.RecordPointer;
import imagemed.types.IMType;
import imagemed.types.flag.IMTypeur;
import imagemed.util.Helper;
import imagemed.util.LocalizedMessage;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.Format;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;



public class DefaultuserValidator extends DBValidator {

	@Override
	protected void preValidate() {
		setMandatoryError( "USID" );
		setMandatoryError( "UTMZ" );

		//ensure all supported plugins have attached records
		String[] allPlugins = PluginToolbox.getAllPlugins();
		String userID = m_rootRec.m_rowID.get( 0 );
		for ( String plugin : allPlugins ) {
			RecordPointer pluginRP = new RecordPointer( PluginToolbox.PLUGINPERUSERTBL, Arrays.asList( new String[]{ userID, plugin } ) );
			if ( m_entry.getRecord( pluginRP ) == null ) {
				LDSRecord child = m_entry.addRecord( PluginToolbox.PLUGINPERUSERTBL, m_rootRec );
				child.setID( Arrays.asList( new String[]{ userID, plugin } ) );
				child.setValue( "PLEN", PluginToolbox.getLimitForPlugin( plugin ) < 0 ? "true" : "false" );
			}
		}

		//Refresh - Set fields
		String refreshingType = getRefreshType();


		if ( REFRESHTYPE_FIELDCHANGE.equals( refreshingType ) ) {
			String refreshingColid = getRefreshingColid();

			// default handling
			if ( "ULDU".equals( refreshingColid ) ) {
				//if default is yes now
				RecordIterator actual = getRefreshingRecords();
				while ( actual.next() ) { // there will be 1
					if ( "yes".equals( actual.getValue( "ULDU" ) ) ) {

						//default = no, for all 
						for ( RecordIterator userLoc = getIterator( m_userLocationTable ); userLoc.next(); ) {
							userLoc.setValue( "ULDU", "no" );
						}

						//ours is put back to yes
						actual.setValue( "ULDU", "yes" );
					}
				}
			}
		}

		// Copy User button
		if ( REFRESHTYPE_BUTTON.equals( refreshingType ) ) {
			if ( "CopyUser".equals( getRefreshingMacroID() ) ) {
				String otherID = m_dataSource.getSession().request.getParameter( "otherID" );
				Helper.DEBUG( "copyuserbutton:" + otherID );
				if ( otherID != null ) {
					try {
						DBLDSEntry other = new DBLDSEntry( (DBLayoutDataSource) m_dataSource );
						other.initFromExisting( otherID );

						// copy the needed fields in root record
						Iterator<String> colids = other.getRootRecord().getColidSet().iterator();
						while ( colids.hasNext() ) {
							String field = colids.next();
							if ( !"USID".equals( field ) && !"UPWD".equals( field ) && !"UPW2".equals( field ) &&
									!"UACS".equals( field ) && !"UACW".equals( field ) && !"UCOD".equals( field ) &&
									!"UVER".equals( field ) && !"UNAM".equals( field ) && !"MAIL".equals( field ) && !"UPH1".equals( field ) && !"UPH2".equals( field ) ) {
								setValue( field, other.getRootRecord().getCurrentValue( field, "" ) );
								Helper.DEBUG( "Field " + field + " set to:" + other.getRootRecord().getCurrentValue( field, "" ) );
							}
						}

						// set plugins - records already exist
						for ( String plugin : allPlugins ) {
							RecordPointer pluginRP = new RecordPointer( PluginToolbox.PLUGINPERUSERTBL, Arrays.asList( new String[]{ userID, plugin } ) );
							LDSRecord child = m_entry.getRecord( pluginRP );
							RecordPointer otherPluginRP = new RecordPointer( PluginToolbox.PLUGINPERUSERTBL, Arrays.asList( new String[]{ otherID, plugin } ) );
							LDSRecord otherChild = other.getRecord( otherPluginRP );

							if ( child != null && otherChild != null ) {
								colids = otherChild.getColidSet().iterator();
								while ( colids.hasNext() ) {
									String field = colids.next();
									if ( !"PLUS".equals( field ) && !"PLNM".equals( field ) ) {
										child.setValue( field, otherChild.getCurrentValue( field, "" ) );
										Helper.DEBUG( "Field " + field + " set to:" + otherChild.getCurrentValue( field, "" ) );
									}
								}

							}
						}

						// set the locations - will not delete/modify the existing ones, if any
						//detect if there's already a default
						boolean hasDef = false;
						for ( RecordIterator userLoc = getIterator( m_userLocationTable ); userLoc.next(); ) {
							if ( "yes".equals( userLoc.getValue( "ULDU" ) ) ) {
								hasDef = true;
							}
						}

						List<LDSRecord> otherLocs = other.getRootRecord().getChildList( m_userLocationTable );
						for ( LDSRecord otherLoc : otherLocs ) {
							LDSRecord newRec = m_entry.addRecord( m_userLocationTable, m_rootRec );
							newRec.setValue( "ULUI", userID );
							colids = otherLoc.getColidSet().iterator();
							while ( colids.hasNext() ) {
								String field = colids.next();
								if ( !"ULID".equals( field ) && !"ULUI".equals( field ) &&
										( !hasDef || !"ULDU".equals( field ) ) ) {
									//if there's a default, we leave the new defaults on 'no'
									newRec.setValue( field, otherLoc.getCurrentValue( field, "" ) );
									Helper.DEBUG( "Field " + field + " set to:" + otherLoc.getCurrentValue( field, "" ) );
								}
							}
						}

						// set the practices - will not delete/modify the existing ones, if any
						List<LDSRecord> otherPracs = other.getRootRecord().getChildList( m_userPracticeTable );
						for ( LDSRecord otherPrac : otherPracs ) {
							LDSRecord newRec = m_entry.addRecord( m_userPracticeTable, m_rootRec );
							newRec.setValue( "UPUI", userID );
							colids = otherPrac.getColidSet().iterator();
							while ( colids.hasNext() ) {
								String field = colids.next();
								if ( !"UPID".equals( field ) && !"UPUI".equals( field ) ) {
									newRec.setValue( field, otherPrac.getCurrentValue( field, "" ) );
									Helper.DEBUG( "Field " + field + " set to:" + otherPrac.getCurrentValue( field, "" ) );
								}
							}
						}

						// set the managed groups - will not delete/modify the existing ones, if any
						List<LDSRecord> otherGroups = other.getRootRecord().getChildList( m_userGroupTable );
						for ( LDSRecord otherGroup : otherGroups ) {
							LDSRecord newRec = m_entry.addRecord( m_userGroupTable, m_rootRec );
							newRec.setValue( "UGUI", userID );
							colids = otherGroup.getColidSet().iterator();
							while ( colids.hasNext() ) {
								String field = colids.next();
								if ( !"UGID".equals( field ) && !"UGUI".equals( field ) ) {
									newRec.setValue( field, otherGroup.getCurrentValue( field, "" ) );
									Helper.DEBUG( "Field " + field + " set to:" + otherGroup.getCurrentValue( field, "" ) );
								}
							}
						}

					}
					catch ( Exception e ) {
						Helper.ERRORLOG( "EXCEPTION:" + Helper.throwable2String( e ) );
					}
				}
			}
		}

	}

	@Override
	public void validate() {
		Helper.DEBUG( "DefaultuserValidator running" );

		String userID = getValue( "USID" );
		if ( !Helper.containsOnly( userID, "0..9a..zA..Z _." ) ) {
			m_errors.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_BADID" ) );
			return;
		}

		// we don't use the general method, because Patrick + Jim wants a special message here (CBT 2194)
		//mustBeUnique( "USID", m_userPage, false );
		String actValue = getRootRec().getCurrentValue( "USID", "" );
		if ( !"".equals( actValue ) ) {
			try {
				Statement st = IMEnv.getDB().createStatement();
				String query = "select * from " + getRootRec().m_stable + " where " +
						IMBaseFieldManager.staticGetDbName( "USID" ) + " = '" + Helper.sqlEncode( actValue ) + "'";
				if ( !getRootRec().isNew() ) {
					query += " AND " + getRootRec().m_IDfieldDbName.get( 0 ) + " != '" + getRootRec().m_rowID.get( 0 ) + "'";
				}
				Helper.DEBUG( "USID query:" + query );
				ResultSet rs = st.executeQuery( query );
				if ( rs != null ) {
					rs.last();
					int numOfResult = rs.getRow();
					rs.beforeFirst();

					if ( numOfResult > 0 ) {
						m_errors.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_EXISTS" ) + " " + actValue );
						while ( rs.next() ) {
							String name = rs.getString( IMBaseFieldManager.staticGetDbName( "UNAM" ) );
							if ( name != null ) {
								IMType type = IMType.getTypeForColid( "UNAM" );
								name = type.displayText( name );
								m_errors.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_USER_NAME" ) + " " + name );
							}
						}
					}
				}
			}
			catch ( Throwable th ) {
				throw new RuntimeException( Helper.throwable2String( th ) );
			}
		}


		// Not filled Location records give a warning
		RecordIterator userLoc = getIterator( m_userLocationTable );
		boolean isValid = false;
		Set<String> locs = new HashSet<String>();
		if ( !userLoc.isEmpty() ) {
			while ( userLoc.next() ) {
				String locID = userLoc.getValue( "ULLI" );
				if ( locID == null || "".equals( locID ) ) {
					m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_LOCATION_SHOULD_BE_SELECTED" ) );
				}
				else {
					for ( RecordIterator loc = userLoc.getIterator( m_locationTable ); loc.next(); ) {
						String del = loc.getValue( "LODD" );
						String name = loc.getValue( "LONA" );
						if ( "yes".equals( del ) ) {
							m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_DELETED_LOCATION" ) + ( name == null ? "" : " " + name ) );
						}
						else {
							isValid = true;
						}
						if ( name != null ) {
							if ( locs.contains( name ) ) {
								m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_MUTIPLE_LOCATION" ) + name );
							}
							locs.add( name );
						}
					}
				}
			}
		}

		// if we have User Manage but no Admin right, then we want some managed groups
		boolean hasGroup = false;
		boolean emptyGroup = false;
		RecordIterator userGroup = getIterator( m_userGroupTable );
		if ( !userGroup.isEmpty() ) {
			while ( userGroup.next() ) {
				String grid = userGroup.getValue( "UGGI" );
				if ( grid != null && !"".equals( grid ) ) {
					hasGroup = true;
				}
				else {
					emptyGroup = true;
				}
			}
		}
		String rights = getValue( "URGT" );
		if ( IMTypeur.rightsIncluded( rights, IMTypeur.USER_MANAGE_RIGHT ) &&
				!IMTypeur.rightsIncluded( rights, IMTypeur.ADMIN_RIGHT + IMTypeur.SUPPORT_RIGHT + IMTypeur.SUPPORT_RIGHT_FOR_SUPPORT_USER ) &&
				!hasGroup ) {
			m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_GROUP_MUST_BE_ADDED" ) );
		}
		else if ( emptyGroup ) {
			m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_GROUP_EMPTY" ) );
		}

		if ( ( !IMTypeur.rightsIncluded( rights, IMTypeur.USER_MANAGE_RIGHT ) ||
				IMTypeur.rightsIncluded( rights, IMTypeur.ADMIN_RIGHT + IMTypeur.SUPPORT_RIGHT + IMTypeur.SUPPORT_RIGHT_FOR_SUPPORT_USER ) ) &&
				hasGroup ) {
			m_warnings.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_GROUP_SHOULD_BE_REMOVED" ) );
		}

		//Saving a User with 'Edit Report' right and without a Location should be forbidden
		// Removed from the default 
		//			if ( !isValid && IMTypeur.rightsIncluded( rights, IMTypeur.REPORT_RIGHT ) ) {
		//				m_errors.add( LocalizedMessage.getSimpleMessage( "USERVALIDATOR_NOLOCATION" ) );
		//			}

		String target = getValue( "USAT" );
		if ( target == null ) {
			target = IMAdminToolbox.AUTH_GRP;
		}
		if ( IMAdminToolbox.AUTH_GRP.equals( target ) ) {
			String group = getValue( "UGRP" );
			try {
				target = IMAdminToolbox.getGroupAuthTarget( group );
			}
			catch ( IMException | SQLException e ) {
				// group missing / should not happen
				throw new RuntimeException( "Getting auth target for group " + group + " failed: " + Helper.toString( e ) );
			}
		}
		if ( IMAdminToolbox.AUTH_SYS.equals( target ) ) {
			target = IMAdminToolbox.getSysAuthTarget();
		}
		if ( IMAdminToolbox.AUTH_EP.equals( target ) ) {
			handlePw();
		}

		Properties plugins = new Properties();
		String[] allPlugins = PluginToolbox.getAllPlugins();
		userID = m_rootRec.m_rowID.get( 0 );
		for ( String plugin : allPlugins ) {
			RecordPointer pluginRP = new RecordPointer( PluginToolbox.PLUGINPERUSERTBL, Arrays.asList( new String[]{ userID, plugin } ) );
			LDSRecord rec = m_entry.getRecord( pluginRP );
			if ( rec != null ) {
				plugins.setProperty( plugin, rec.getCurrentValue( "PLEN", "false" ) );
			}
		}

		try {
			List<String> excPlugins = PluginToolbox.checkPluginChangeFor( userID, plugins );
			for ( String excPlugin : excPlugins ) {
				m_errors.add( LocalizedMessage.getCompoundMessage( "EXCEPTION_LICENSE_EXCEEDED", new Format[]{ null }, new Object[]{ excPlugin }, false ) );
			}
		}
		catch ( SQLException e ) {
			throw new RuntimeException( "Validating plugins configuration failed: " + Helper.toString( e ) );
		}

	}

}
